#! /bin/env python

import os, sys
import produtil.log, produtil.setup, produtil.run
import hwrf_expt

from produtil.log import jlogger
from produtil.run import mpirun, mpi, checkrun

def fail(msg):
    jlogger.error(msg)
    sys.exit(2)

def launchself():
    os.environ['SCR_IMMEDIATE_EXIT']='YES'
    os.environ['LAUNCH_SELF']='NO'
    checkrun(mpirun(mpi('mpiserial')[os.path.realpath(__file__)],allranks=True),sleeptime=10)

def run_relocate():
    irank=int(os.environ['SCR_COMM_RANK'])
    isize=int(os.environ['SCR_COMM_SIZE'])
    print 'GREETINGS FROM RANK '+repr(irank)
    itime=0
    my_time=None
    my_obj=None

    hwrf_expt.init_module()

    for fgat_time,fgat_obj in hwrf_expt.fgat_init.fhr_and_init():
        if itime==irank:
            my_time=fgat_time
            my_obj=fgat_obj
        itime+=1

    del fgat_time, fgat_obj

    print 'rank %d will run time %s, object %s'%(irank,repr(my_time),repr(my_obj))

    if itime!=isize:
        jlogger.critical('ABORT: MPI_COMM_WORLD size is %d but there are %d fgat times.'%(isize,itime))
    assert(my_time is not None)
    try:
        my_obj.run_relocate()
    except Exception as e:
        my_obj.log().critical('ABORT: fgat relocate for time %s failed: %s'
            %(repr(fgat_time),str(e)),exc_info=True)
        sys.exit(2)

def main():
    print 'HELLO THERE'
    produtil.setup.setup()
    ENV=os.environ
    init_model=ENV['INIT_MODEL'].lower()
    if init_model!='gdas1':
        fail('Aborting: init_model=%s must be gdas1 for the parallel relocate.  Use exhwrf_relocate.py for the GFS relocate.'%(repr(init_model),))
    if ENV.get('LAUNCH_SELF','YES')=='YES':
        launchself()
    else:
        run_relocate()

if __name__=='__main__':
    main()
