#! /bin/env python

import logging, os
import produtil.sigsafety, produtil.cd, produtil.run, produtil.setup
import produtil.log
import hwrf_expt
import hwrf_alerts

from produtil.log import jlogger
from produtil.cd import NamedDir
from produtil.run import mpi, mpirun, checkrun

def gribber():
    jlogger.info(hwrf_expt.conf.strinterp('config',
            '{stormlabel}: starting regribbing job for {out_prefix}'))
    with NamedDir(hwrf_expt.WORKhwrf,logger=logging.getLogger()) as t:
        hwrf_expt.gribber.uncomplete()
        #hwrf_expt.gribber.unrun()
        hwrf_expt.gribber.run()
    jlogger.info(hwrf_expt.conf.strinterp('config',
            '{stormlabel}: completed regribbing job for {out_prefix}'))

def tracker():
    jlogger.info(hwrf_expt.conf.strinterp('config',
            '{stormlabel}: starting tracker job for {out_prefix}'))
    with NamedDir(hwrf_expt.WORKhwrf,logger=logging.getLogger()) as t:
        hwrf_expt.tracker.run()
    jlogger.info(hwrf_expt.conf.strinterp('config',
            '{stormlabel}: completed tracker job for {out_prefix}'))

def products():
    jlogger.info(hwrf_expt.conf.strinterp('config',
            '{stormlabel}: starting nhc_products job for {out_prefix}'))
    with NamedDir(hwrf_expt.WORKhwrf,logger=logging.getLogger()) as t:
        hwrf_expt.nhcp.run()
    jlogger.info(hwrf_expt.conf.strinterp('config',
            '{stormlabel}: completed nhc_products job for {out_prefix}'))

def gribber_tracker():
    """This is run multiple times in parallel.  One copy runs the tracker,
while the rest run the regribbing."""
    rank=int(os.environ['SCR_COMM_RANK'])
    count=int(os.environ['SCR_COMM_SIZE'])
    print 'MPI communicator: rank=%d size=%d'%(rank,count)
    hwrf_expt.init_module()
    hwrf_alerts.add_regrib_alerts()
    hwrf_alerts.add_tracker_alerts()
    subdict={ 'RANK':rank, 'COUNT':count, 'WHO':'regribber', 
              'jobid':produtil.batchsystem.jobid(), 
              'WORKhwrf':hwrf_expt.conf.getdir('WORKhwrf') }
    if(count<2):
        # Cannot run gribber and tracker in parallel with fewer than
        # two ranks.  Run gribber, then tracker:
        print 'ONE MPI RANK: RUN GRIBBER THEN TRACKER'
        gribber()
        tracker()
    elif(rank==0):
        # More that one rank available.  First rank runs tracker. 
        print 'MPI, FIRST RANK: RUN TRACKER'

        # Redirect stdout and stderr
        if 'TRACKER_LOGS' in os.environ:
            r=os.environ.get('TRACKER_LOGS')
        else:
            r=hwrf_expt.conf.strinterp(
                'config','%(WORKhwrf)s/%(jobid)s-tracker.log')
        rstdout=r % dict(subdict, WHO='tracker', STREAM='out')
        rstderr=r % dict(subdict, WHO='tracker', STREAM='err')
        produtil.log.mpi_redirect(stdoutfile=rstdout,stderrfile=None,
                                  threadname='tracker')

        # Run the tracker
        tracker()
    else:
        # More than one rank available.  Ranks after first run
        # gribber.
        print 'MPI, NOT FIRST RANK: RUN GRIBBER'
        # Redirect stdout and stderr
        if 'REGRIBBER_LOGS' in os.environ:
            r=os.environ['REGRIBBER_LOGS']
        else:
            r=hwrf_expt.conf.strinterp(
                'config',
                '%(WORKhwrf)s/%(jobid)s-regribber-%(RANK)04d.%(STREAM)s')
        rstdout=r % dict(subdict, WHO='regribber', STREAM='out')
        rstderr=r % dict(subdict, WHO='regribber', STREAM='err')
        logging.getLogger('hwrf').warning(
            'Redirecting regribber %d to: stderr=%s stdout=%s'%
            ( rank, rstderr, rstdout ))
        produtil.log.mpi_redirect(stdoutfile=rstdout,stderrfile=rstderr,
                                  threadname='regrib%d'%(rank,))

        # Run the gribber
        gribber()

def launchself():
    # Instruct further processes not to re-launch scripts via mpirun:
    os.environ['LAUNCH_SELF']='no'
    # Launch four copies of myself.  We must use mpiserial for this
    # because we need the SCR_COMM_RANK variable:
    checkrun(mpirun(mpi(hwrf_expt.conf.getexe('mpiserial'))[os.path.realpath(__file__)],allranks=True))
    # Calling checkrun ensures the program exits abnormally if
    # mpirun.lsf (or whatever you use) exits abnormally.


def doit(): 
    produtil.setup.setup()
    if 'SCR_COMM_RANK' not in os.environ \
            and os.environ.get('LAUNCH_SELF','yes')=='yes':
        # This is the top level of the job: we are NOT inside an
        # mpi_serial call.

        # Initialize the hwrf_expt and re-call any callbacks for completed products:
        hwrf_expt.init_module()
        logger=logging.getLogger('exhwrf_products')
        hwrf_alerts.add_nhc_alerts()
        hwrf_alerts.add_regrib_alerts()
        hwrf_alerts.add_wave_alerts()
        logger.warning('''Rerunning dbn_alert for prior jobs' posted files.''')
        hwrf_expt.gribber.call_completed_callbacks()

        # We're in the top-level job.  Launch copies of ourself to run the
        # gribber and tracker:
        logger.warning('---------------------------------------------------')
        logger.warning('LAUNCH PARALLEL PORTION OF SCRIPT------------------')
        logger.warning('---------------------------------------------------')
        launchself()
        logger.warning('---------------------------------------------------')
        logger.warning('PARALLEL PORTION OF SCRIPT HAS ENDED---------------')
        logger.warning('---------------------------------------------------')

        # Gribber and tracker succeeded.  Run the products job:
        products()
    else:
        # We're in a subprocess.  Just run the gribber and tracker and return:
        gribber_tracker()

#cProfile.run('doit()')
doit()
