#! /usr/bin/env python

import os.path, sys, glob, logging, collections
import hwrf.numerics, hwrf.revital

from hwrf.numerics import to_datetime, to_timedelta

def check_gfs(path,cycle):
    hh=int(cycle[8:10])
    files= \
        ['gfs.t%02dz.sf%02d' % (hh, x*6) for x in xrange(22)] + \
        ['gfs.t%02dz.pgrbf00'%(hh,),  'gfs.t%02dz.abias'%(hh,),
         'gfs.t%02dz.sanl'%(hh,),     'gfs.t%02dz.sfcanl'%(hh,),
         'gfs.t%02dz.syndata.tcvitals.tm00'%(hh,),
         'avn.t%02dz.cyclone.trackatcfunix'%(hh,),
         'gfs.t%02dz.satang'%(hh,),   'gfs.t%02dz.pgrb2f00'%(hh,),
         'gfs.t%02dz.sf09'%(hh,)]

    bufr_d=[x for x in glob.glob('%s/gfs.t%02dz.*.bufr_d'%(path,hh))]
    bufr_d.append('%s/gfs.t%02dz.atms.tm00.bufr_d'%(path,hh,))

    okay=True
    for basename in files:
        checkme=os.path.join(path,basename)
        if not os.path.exists(checkme):
            print '%s: %s: does not exist'%(path,basename,)
            ok=False
        elif not os.path.isfile(checkme):
            print '%s: %s: not a regular file'%(path,basename,)
            ok=False
        elif not 'syndata.tcvitals.tm00' in basename:
            size=os.path.getsize(checkme)
            if size<30:
                print '%s: %s: implausably small (%d bytes)'%(path,basename,size)
                ok=False

    for filename in bufr_d:
        if not os.path.exists(filename):
            print '%s: %s: does not exist'%(path,os.path.basename(filename),)
            ok=False
        elif not os.path.isfile(filename):
            print '%s: %s: not a regular file'%(path,os.path.basename(filename),)
            ok=False

    if not okay:
        print '%s: some files missing or empty'
    elif len(bufr_d)<10:
        print '%s: only %d bufr_d files'%(path,len(bufr_d))
        okay=False
    elif okay:
        print '%s: looks good'%(path,)
    return okay

def check_enkf(path,cycle):
    #sfg_2013090300_fhr06s_mem036
    sfsize=100796456
    ok=True
    for sfile in ['sfg_%s_fhr06s_mem%03d'%(cycle,1+x) for x in xrange(80)]:
        checkme=os.path.join(path,sfile)
        if not os.path.exists(checkme):
            print '%s: %s: does not exist'%(path,sfile)
            ok=False
        elif os.path.getsize(checkme)<sfsize:
            print '%s: %s: less than %d bytes'%(path,sfile,sfsize)
            ok=False
    if ok:
        print '%s: looks good'%(path,)
    return ok

def check_gdas1(path,cycle):
    hh=int(cycle[8:10])
    files=['gdas1.t%02dz.abias'%(hh,),
           'gdas1.t%02dz.satang'%(hh,)]
    files.extend(['gdas1.t%02dz.sf%02d'%(hh,x+3) for x in xrange(7)])
    files.extend(['gdas1.t%02dz.pgrbh%02d'%(hh,x+3) for x in xrange(7)])
    ok=True
    for gfile in files:
        checkme=os.path.join(path,gfile)
        if not os.path.exists(checkme):
            print '%s: %s: does not exist'%(path,gfile)
            ok=False
        elif os.path.getsize(checkme)<1:
            print '%s: %s: file is empty'%(path,gfile)
            ok=False
    if ok:
        print '%s: looks good'%(path,)
    return ok

def locate_dir(what,when):
    when=to_datetime(when)
    parents={
        'GFS':[
            '/pan2/projects/hwrf-vd/hwrf-input/HISTORY/GFS.%d'%(when.year,),
            '/scratch1/portfolios/NCEPDEV/hwrf/noscrub/input/HISTORY/GFS.%d'%(when.year,),
            '/hwrf/noscrub/input/HISTORY/GFS.%d'%(when.year,)],
        'GDAS1':[
            '/pan2/projects/hwrf-vd/hwrf-input/GDAS1/%d'%(when.year,),
            '/scratch1/portfolios/NCEPDEV/hwrf/noscrub/input/GDAS1/%d'%(when.year,),
            '/hwrf/noscrub/input/GDAS1/%d'%(when.year,)],
        'ENKF':[
            '/pan2/projects/hwrf-vd/hwrf-input/ENKF/%d'%(when.year,),
            '/scratch1/portfolios/NCEPDEV/hwrf/noscrub/input/ENKF/%d'%(when.year,),
            '/hwrf/noscrub/input/ENKF/%d'%(when.year,)]   }
    for parent in parents[what]:
        if os.path.isdir(parent):
            return os.path.join(parent,when.strftime('%Y%m%d%H'))

def main():
    global_okay=True
    logging.basicConfig(stream=sys.stderr)
    logger=logging.getLogger()
    vitals=hwrf.revital.Revital(logger=logger)
    vitals.readfiles(sys.argv[1:])
    vitals.renumber()
    cycles=set()
    cyclewhy=collections.defaultdict(set)
    for vital in vitals.vitals:
        if vital.stnum>0 and vital.stnum<90 and vital.lat>=0 and vital.basin1.upper() in 'LEC':
            when=vital.when.strftime('%Y%m%d%H')
            cycles.add(when)
            cyclewhy[when].add(vital.stormid3.upper())
    for cycle in sorted(cycles):
        when=to_datetime(cycle)
        prev=(when-to_timedelta('06:00')).strftime('%Y%m%d%H')
        print '%s: check this; needed for %s' % \
            (when,', '.join(cyclewhy[cycle]))
        #print 'GRAB GFS=%s GDAS1/ENKF=%s'%(cycle,prev)
        gfsdir=locate_dir('GFS',when)
        gdasdir=locate_dir('GDAS1',prev)
        enkfdir=locate_dir('ENKF',prev)
        #print 'GFS in %s'%(gfsdir,)
        #print 'ENKF in %s'%(enkfdir,)
        #print 'GDAS in %s'%(gdasdir,)
        gfsokay = check_gfs(gfsdir,cycle)
        gdasokay = check_gdas1(gdasdir,prev)
        enkfokay = check_enkf(enkfdir,prev)
        if gfsokay: 
            print '%s: GFS okay'%(cycle,)
        else:
            print '%s: GFS data missing'%(cycle,)
        if enkfokay: 
            print '%s: ENKF okay'%(cycle,)
        else:
            print '%s: ENKF data missing'%(cycle,)
        if gdasokay: 
            print '%s: GDAS okay'%(cycle,)
        else:
            print '%s: GDAS data missing'%(cycle,)
        global_okay = global_okay and gfsokay and gdasokay and enkfokay
    sys.exit(0 if global_okay else 1)

if __name__ == "__main__":
    main()
