#! /usr/bin/ksh
###############################################################
# hwrf_output_email.sh
#################
# This script emails the GFDL/HWRF model output files to the SDM.
#      GFDL file: whxx04.${cycle}.${stormenv}
#      HWRF file: ${stormenv}.yyyymmddhh.afos
# Usage: sh hwrf_output.sh $RUN $INFILE
#        where RUN=hur for GFDL runs or hwrf for HWRF runs
#        where INFILE=GFDL file as listed above or HWRF file as listed above.
# Jun 2011  Kelly Kempisty
###############################################################

set -xa

## INPUT VARIABLES ##

if [ $# -eq 2 ]
then
   RUN=$1
   INFILE=$2
fi

echo "RUN = ${RUN}"

if [ ${RUN} = hwrf ];
then
   echo $INFILE | sed 's/\./ /g' > sdm_stormenv
   awk '{ print $1 }' sdm_stormenv > sdm_storminfo
elif [ ${RUN} = hur ];
then
   echo $INFILE | sed 's/\./ /g' > sdm_stormenv
   awk '{ print $3 }' sdm_stormenv > sdm_storminfo
fi
storm_info=`cat sdm_storminfo`
storm_subject=`echo $storm_info | sed 's/[0-9]*[a-z]$/ &/g'`
storm_name=`echo $storm_subject | awk '{ print $1 }' | perl -pe 's/^(.{0})(.)/$1\U$2/;'`
storm_number=`echo $storm_subject | awk '{ print $2 }' | tr [:lower:] [:upper:]`
if [ `echo $storm_number | cut -c3` = "E" ];
then
   basin="Eastern Pacific"
elif [ `echo $storm_number | cut -c3` = "C" ];
then
   basin="Central Pacific"
else
   basin="Atlantic"
fi
MACHINE=`hostname | cut -c1`
if [ ${RUN} = hwrf ];
then 
   ssh dbnet_prod${MACHINE} "cat $COMOUT/$INFILE | mail -s \"${cyc}Z HWRF Output for $basin Tropical System $storm_name ($storm_number)\" sdm@noaa.gov"
elif [ ${RUN} = hur ];
then
   ssh dbnet_prod${MACHINE} "cat $COMOUT/$INFILE | mail -s \"${cyc}Z GFDL Output for $basin Tropical System $storm_name ($storm_number)\" sdm@noaa.gov"
fi
echo `hostname`
