#! /bin/env python

import logging, os, sys
import produtil.cd, produtil.datastore, produtil.locking, produtil.fileop
import hwrf.numerics, hwrf.pullgfs

from produtil.datastore import Product, Datastore
from produtil.locking import LockFile, LockHeld
from hwrf.numerics import to_datetime, to_datetime_rel,to_timedelta

def doit(start,stop,dt,rel,pull_all=False):
    startdir=os.getcwd()
    handler=logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter('pullgfs:%(levelname)s:%(asctime)s: %(message)s',
                          '%Y%m%d.%H%M%S'))
    logger=logging.getLogger('pullgfs')
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)

    lockdir=os.path.join(startdir,'locks')
    produtil.fileop.makedirs(lockdir)

    ds=Datastore(os.path.join(startdir,'completion.sqlite3'),logger=logger)
    start=to_datetime(start)
    stop=to_datetime_rel(stop,start)
    dt=to_timedelta(dt)
    when=start
    while when<=stop:
        tempname=when.strftime('pull.%Y%m%d%H.')
        pn=when.strftime('gfs.%Y%m%d%H')
        product=produtil.datastore.Product(ds,prodname=pn,category='pullgfs')
        lockfile=os.path.join(lockdir,'%s.lock'%(pn,))
        if product.available:
            logger.info('%s: already done.'%(when.strftime('%Y%m%d%H'),))
            when+=dt
            continue
        try:
            with LockFile(lockfile,max_tries=2,sleep_time=2):
                with produtil.cd.TempDir(prefix=tempname,suffix='.tmpdir',
                                              logger=logger):
                    p=hwrf.pullgfs.PullGFSInput(
                        when=when,
                        gfstarget=os.path.join(rel,'HISTORY/GFS.%Y/%Y%m%d%H/'),
                        enkftarget=os.path.join(rel,'ENKF/%Y/%Y%m%d%H/'),
                        gdastarget=os.path.join(rel,'GDAS1/%Y/%Y%m%d%H/'),
                        logger=logger)
                    if pull_all:
                        p.process_all()
                    else:
                        p.process_gdas_and_gfs_anl()
            product.available=True
            logger.critical('%s: REJOICE! Pulled all data for this cycle! REJOICE!'%
                            (when.strftime('%Y%m%d%H'),))
        except LockHeld as lh:
            logger.warning('%s: lock held; moving on.'%(when.strftime('%Y%m%d%H'),))
        except Exception as e:
            logger.error('%s: %s'%(when.strftime('%Y%m%d%H'),str(e)),
                              exc_info=True)
        finally:
            when+=dt

if sys.argv[1]=='--all':
    doit(sys.argv[3],sys.argv[4],'06:00',sys.argv[2],pull_all=True)
else:
    doit(sys.argv[2],sys.argv[3],'06:00',sys.argv[1],pull_all=False)
