#! /usr/bin/env bash
set -x

if [[ -z "$WHERE_AM_I" ]] ; then
    export WHERE_AM_I=$( $USHhwrf/hwrf_where_am_i.sh )
fi

if [[ "$WHERE_AM_I" == jet && -z "$JET_NAME" ]] ; then
    export JET_NAME=${PARCLASSLL:0:4}
fi

if [[ "$1" == "--wcoss-remote-slave" ]] ; then
    # We are a slave process on a login node, running because a
    # compute node has sshed here to submit a job.
    . /usrx/local/Modules/default/init/bash
    module load lsf
    module use /hwrf/noscrub/soft/modulefiles
    module load ruby emc-utils

    infile=$2
    tempfile=$3.in
    outfile=$3

    exec emcsub --copy "$outfile" < "$tempfile"
elif [[ "$1" == "--direct-submit" ]] ; then
    shift 1
    unset ssh_remote_server
fi

if [[ "$#" != 2 ]] ; then
    cat<<EOF 1>&2
$0: You have called hwrf_submit.sh incorrectly.  

    Syntax: hwrf_submit.sh INPUT_SCRIPT.shell OUTPUT_SCRIPT.sh

This script reads in an input .shell script, parses it with
\${USHhwrf}/hwrf_stream_parse.pl and then writes out the result in
OUTPUT_SCRIPT.sh.  Next, it submits OUTPUT_SCRIPT.sh to LoadLeveler
via llsubmit.

You MUST specify BOTH arguments: the INPUT_SCRIPT.shell and the
OUTPUT_SCRIPT.sh.  You provided $# arguments.  Aborting.
EOF

    exit 1
fi

function qsub_stdin {
    qsub < "$1"
}

function ssh_emcsub_stdin {
    ssh "$ssh_remote_server" "$USHhwrf/hwrf_submit.sh" --wcoss-remote-slave "$infile" "$outfile" < /dev/null
}

function emcsub_stdin {
    emcsub --copy "$outfile" < "$tempfile"
}

ATCFID="${ATCFID:-$STID}"

# Try to figure out the year:
YEAR="${YEAR:-$YYYY}"
YEAR="${YEAR:-${YMDH:0:4}}"
YEAR="${YEAR:-${INPYMDH:0:4}}"

# Try to figure out the YMDH:
YMDH="${YMDH:-${YYYYMMDDHH:-${INPYMDH:-}}}"

infile=$1
tempfile=$2.in
outfile=$2
name=$( basename "$2" )

set +x
# Try to figure out the name and id of the job we're being run from.
MY_JOB_NAME="(unknown)"
MY_JOB_NAME="${LOADL_STEP_NAME:-$MY_JOB_NAME}"
MY_JOB_NAME="${PBS_JOBNAME:-$MY_JOB_NAME}"
MY_JOB_NAME="${LSB_JOBNAME:-$MY_JOB_NAME}"

MY_JOB_ID="(unknown)"
MY_JOB_ID="${LOADL_STEP_ID:-$MY_JOB_ID}"
MY_JOB_ID="${PBS_JOBID:-$MY_JOB_ID}"
MY_JOB_ID="${LSB_JOBID:-$MY_JOB_ID}"
set -x

# if [[ ! -z "$ATCFID" && ! -z "$SUBEXPT" && "$WHERE_AM_I" == jet ]] ; then
#     eval $( $USHhwrf/hwrf_reservation.sh $ATCFID $SUBEXPT $YEAR )
# elif [[ ! -z "$ATCFID" && ! -z "$EXPT" && "$WHERE_AM_I" == jet ]] ; then
#     eval $( $USHhwrf/hwrf_reservation.sh $ATCFID $EXPT $YEAR )
# fi

if [[ ! -z "$ATCFID" && ! -z "$SUBEXPT" && "$WHERE_AM_I" == jet ]] ; then
    eval $( $USHhwrf/hwrf_reservation.pl $ATCFID $SUBEXPT ${YMDH:-$YEAR} "$infile" )
elif [[ ! -z "$ATCFID" && ! -z "$EXPT" && "$WHERE_AM_I" == jet ]] ; then
    eval $( $USHhwrf/hwrf_reservation.pl $ATCFID $EXPT ${YMDH:-$YEAR} "$infile" )
fi

if [[ ! -x "${USHhwrf}/hwrf_stream_parse.pl" ]] ; then
    echo "$0: File \"${USHhwrf}/hwrf_stream_parse.pl\" is not a file or is not executable.  Aborting." 1>&2
    exit 2
fi

${USHhwrf}/hwrf_stream_parse.pl $infile $tempfile
if [[ "$?" != 0 ]] ; then
    echo "$0: Program \"${USHhwrf}/hwrf_stream_parse.pl\" returned non-zero exit value $?.  Aborting." 1>&2
    exit 3
fi

if [[ "$WHERE_AM_I" == jet ]] ; then
    if ( ! /usr/bin/which emcsub || ! echo $RUBYLIB | grep -i emc-utils > /dev/null ) ; then
        # We must find the emcsub command.
        for mf in \
            /apps/Modules/${MODULE_VERSION:-3.2.9}/init/bash \
            /opt/modules/Modules/${MODULE_VERSION:-3.2.6}/init/bash ; do
            if [[ -s "$mf" ]] ; then
                . "$mf"
                module use /lfs1/projects/hwrf-vd/emc-utils/modulefiles
                module load emc-utils
            fi
        done
    fi
    if ( ! /usr/bin/which emcsub > /dev/null ) ; then
        echo "$0: cannot find emcsub.  Aborting." 1>&2
        exit 19
    fi
    exe=emcsub_stdin
elif [[ "$WHERE_AM_I" == ttfri ]] ; then
    if ( ! /usr/bin/which emcsub || ! echo $RUBYLIB | grep -i emc-utils > /dev/null ) ; then
            . /opt/modules/Modules/3.2.9/init/ksh
            module load mvapich2-1.8/intel-12.1
            module load netcdf-4.1.3/intel-12.1
            module load ncl_ncarg/6.0.0
            module load opengrads/2.0.1.oga.1
            module load nco/4.2.1
            module load cdo/1.5.6.1
	    module use /home/emcuser/modulefiles
	    module load emc-utils
    fi
    exe=emcsub_stdin
elif [[ "$WHERE_AM_I" == zeus ]] ; then
    if ( ! /usr/bin/which emcsub || ! echo $RUBYLIB | grep -i emc-utils > /dev/null ) ; then
        # We must find the emcsub command.
        . /usr/share/Modules/init/bash
        module use /contrib/emc-utils/versions/1.0.0/modulefiles
        module load emc-utils
    fi
    if ( ! /usr/bin/which emcsub > /dev/null ) ; then
        echo "$0: cannot find emcsub.  Aborting." 1>&2
        exit 19
    fi
    exe=emcsub_stdin
elif [[ "$WHERE_AM_I" == wcoss ]] ; then
    # Make sure we have emcsub
    . /usrx/local/Modules/default/init/bash
    if ( ! /usr/bin/which bsub > /dev/null ) ; then
        module load lsf
    fi
    if ( ! /usr/bin/which emcsub || ! echo $RUBYLIB | grep -i emc-utils > /dev/null ) ; then
        module use /hwrf/noscrub/soft/modulefiles
        module load ruby emc-utils
    fi

    # On WCOSS compute nodes, we cannot submit jobs reliably
    # so we ssh to a login node ($ssh_remote_server), but
    # if we're on a login node, we submit jobs directly 
    # (ssh_remote_server='')
    # IBM is working on a fix for that.
    if [[ "${ssh_remote_server:-}" != '' ]] ; then
        exe=ssh_emcsub_stdin
    else
        exe=emcsub_stdin
    fi
else
    # We're on CCS.  Make sure we have ruby and emcsub.
    if ( ! /usr/bin/which ruby ) ; then
        export PATH=/usrx/local/ruby-1.8.6/bin/:$PATH
    fi
    if ( ! /usr/bin/which emcsub > /dev/null 2>&1 ) ; then
        # Need to find emc-utils
        export PATH=/u/wx20st/emc-utils-1.0.0/bin:$PATH
    fi
    if ( ! echo $RUBYLIB | grep -i emc-utils > /dev/null 2>&1 ) ; then
        # Need to add the ruby stuff for emc-utils to the $RUBYLIB:
        export RUBYLIB=/u/wx20st/emc-utils-1.0.0/lib${RUBYLIB:+:$RUBYLIB}
    fi
    exe=emcsub_stdin
fi

if [[ ! -z "$DATA" && -d "$DATA" ]] ; then
    if [[ ! -d "$DATA/job-ids/" ]] ; then
        "$USHhwrf/hwrf_mkdirp.sh" "$DATA/job-ids"
    fi
    jobfilebase="$DATA/job-ids/$name-$( date +%s )-$$"
    jobfile="$jobfilebase.jobid"

    cat<<EOF >> "$DATA/submit.out" 2>&1
========================================================================
== my job id=$MY_JOB_ID name=$MY_JOB_NAME
== time = $( date ) on host $( hostname )
== hwrf_submit $infile $outfile
EOF
    ( $exe "$outfile" | tee "$jobfile" ) 2>&1 | tee -a "$DATA"/submit.out
    ret="$?"
else
    "$exe" "$outfile"
    ret="$?"
fi

if [[ "$ret" == 999 ]] ; then
    # Special value 999 means no job id was produced
    exit 5
fi

if [[ "$ret" != 0 ]] ; then
    echo "$0: $exe $2 returned a non-zero exit status $ret.  Aborting."  1>&2
    exit 4
fi

exit 0
